<?php

use \gincore\Api;
use \gincore\Webhook;
use \gincore\Categories;
use \gincore\Products;
use \gincore\Settings;

class ControllerExtensionModuleGincore extends Controller
{

    /**
     * 
     */
    public function index()
    {
        $this->load->language('extension/module/gincore');
        $this->document->setTitle($this->language->get('heading_title'));
        
        $this->load->model('extension/module/gincore');
        $this->load->model('localisation/language');
        $this->load->model('localisation/stock_status');
        $settings = (new Settings($this->registry))->getSettings();
        $webhook = new Webhook($settings, $this->registry);
        $categories = new Categories($settings, $this->registry);
        $products = new Products($settings, $this->registry);
        
        if(isset($this->session->data['success'])){
            $success = $this->session->data['success'];
            unset($this->session->data['success']);
        }
        if(isset($this->session->data['error'])){
            $error = $this->session->data['error'];
            unset($this->session->data['error']);
        }
        
        $this->response->setOutput(
            $this->load->view('extension/module/gincore', [
                '_' => $this->language->all(),
                'settings' => $settings,
                'success' => !empty($success) ? $success : '',
                'error' => !empty($error) ? $error : '',
                'breadcrumbs' => $this->getBreadCrumbs(),
                'header' => $this->load->controller('common/header'),
                'column_left' => $this->load->controller('common/column_left'),
                'footer' => $this->load->controller('common/footer'),
                'webhook' => $webhook->getWebhookUrl(),
                'action' => $this->url->link('extension/module/gincore/save', $this->getUserTokenUrlPart(), true),
                'cancel' => $this->url->link('marketplace/extension', $this->getUserTokenUrlPart().'&type=module', true),
                'import_categories_link' => $this->url->link('extension/module/gincore/importCategories', $this->getUserTokenUrlPart(), true),
                'update_categories_link' => $this->url->link('extension/module/gincore/updateCategories', $this->getUserTokenUrlPart(), true),
                'remove_categories_link' => $this->url->link('extension/module/gincore/removeCategories', $this->getUserTokenUrlPart(), true),
                'import_products_link' => $this->url->link('extension/module/gincore/importProducts', $this->getUserTokenUrlPart(), true),
                'update_products_link' => $this->url->link('extension/module/gincore/updateProducts', $this->getUserTokenUrlPart(), true),
                'remove_products_link' => $this->url->link('extension/module/gincore/removeProducts', $this->getUserTokenUrlPart(), true),
                'imported_categories' => $categories->getImportedCategoriesCount(),
                'imported_products' => $products->getImportedProductsCount(),
                'languages' => $this->model_localisation_language->getLanguages(),
                'statuses' => $this->model_localisation_stock_status->getStockStatuses(),
            ])
        );
    }
    
    /**
     * 
     */
    public function save()
    {
        $this->load->language('extension/module/gincore');
        
        if(!$this->user->hasPermission('modify', 'extension/module/gincore')){
            $this->session->data['error'] = $this->language->get('error_permission');
            $this->redirectToIndex(); 
        }
        $this->load->model('extension/module/gincore');
        
        $post_settings = $this->model_extension_module_gincore->prepareSettingsFromPost($this->request->post);
        if(!empty($post_settings['enabled'])){
            $api = new Api($post_settings);
            if(!$api->apiTest()){
                $this->session->data['error'] = $this->language->get('error_api_not_reachable');
                $this->redirectToIndex(); 
            }
        }
        
        $this->model_extension_module_gincore->savePreparedSettings($post_settings);
        $this->session->data['success'] = $this->language->get('success_settings_saved');
        $this->redirectToIndex(); 
    }
    
    /**
     * @return array
     */
    protected function getBreadCrumbs()
    {
        $breadcrumbs = [];
        $breadcrumbs[] = [
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', $this->getUserTokenUrlPart(), true)
        ];
        $breadcrumbs[] = [
            'text' => $this->language->get('text_extensions'),
            'href' => $this->url->link('marketplace/extension', $this->getUserTokenUrlPart() . '&type=module', true),
        ];
        $breadcrumbs[] = [
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/module/gincore', $this->getUserTokenUrlPart(), true),
        ];
        return $breadcrumbs; 
    }
    
    /**
     * 
     */
    protected function redirectToIndex($tab = '')
    {
        $this->response->redirect($this->url->link('extension/module/gincore', $this->getUserTokenUrlPart() . $tab, true));
    }
    
    /**
     * @return Api
     */
    private function api()
    {
        $settings = new Settings($this->registry);
        return new Api($settings->getSettings());
    }
    
    /**
     * @param $update
     */
    public function importCategories($update = false)
    {
        $settings = (new Settings($this->registry))->getSettings();
        $categories = new Categories($settings, $this->registry);
        
        $this->api()->getCategoriesEach(function($response) use ($update, $categories){
            if(isset($response['categories'])){
                foreach($response['categories'] as $category){
                    if($categories->hasCategory($category['id'])){
                        if($update){
                            $categories->updateCategory($category);
                        }
                        continue;
                    }
                    $categories->createCategory($category);
                }
            }
        });
        $categories->repairCategories();
        
        $this->redirectToIndex('#tab-import');
    }
    
    /**
     * 
     */
    public function updateCategories()
    {
        $this->importCategories(true);
    }
    
    /**
     * 
     */
    public function removeCategories()
    {
        $settings = (new Settings($this->registry))->getSettings();
        $categories = new Categories($settings, $this->registry);
        
        $categories->removeCategories();
        
        $this->redirectToIndex('#tab-import');
    }
    
    /**
     * 
     */
    public function importProducts($update = false)
    {
        $settings = (new Settings($this->registry))->getSettings();
        $products = new Products($settings, $this->registry);
        
        $this->api()->getProductsEach(function($response) use ($update, $products){
            if(isset($response['products'])){
                foreach($response['products'] as $product){
                    if($products->hasProduct($product['id'])){
                        if($update){
                            $products->updateProduct($product);
                        }
                        continue;
                    }
                    $products->createProduct($product);
                }
            }
            
        });
        
        $this->redirectToIndex('#tab-import');
    }
    
    /**
     * 
     */
    public function updateProducts()
    {
        $this->importProducts(true);
    }
    
    /**
     * 
     */
    public function removeProducts()
    {
        $settings = (new Settings($this->registry))->getSettings();
        $products = new Products($settings, $this->registry);
        
        $products->removeProducts();
        
        $this->redirectToIndex('#tab-import');
    }
    
    /**
     * 
     */
    public function install() 
    {
        $this->load->model('extension/module/gincore');
        $this->model_extension_module_gincore->install();
    }

    /**
     * 
     */
    public function uninstall() 
    {
        $this->load->model('extension/module/gincore');
        $this->model_extension_module_gincore->uninstall();
    }
    
    /**
     * @return string
     */
    public function getUserTokenUrlPart()
    {
        if(strpos(VERSION, '2.') === 0){
            return 'token='.$this->session->data['token'];
        }else{ // >=3
            return 'user_token='.$this->session->data['user_token'];
        }
    }
    
}
