<?php

class ModelExtensionModuleGincore extends Model
{

    /**
     * @param array $post
     * @return array
     */
    public function prepareSettingsFromPost($post)
    {
        $this->load->model('setting/setting');
        
        $current_settings = $this->model_setting_setting->getSetting('gincore');
        $current = $current_settings ? $current_settings['gincore_settings'] : [];
        
        $settings = array_merge($current, [
            'enabled' => !empty($post['gincore_enabled']) ? 1 : 0,
            'api_domain' => !empty($post['gincore_api_domain']) ? $post['gincore_api_domain'] : '',
            'api_key' => !empty($post['gincore_api_key']) ? $post['gincore_api_key'] : '',
            'language_id' => !empty($post['gincore_language_id']) ? $post['gincore_language_id'] : 0,
            'stock_status_id' => !empty($post['gincore_stock_status_id']) ? $post['gincore_stock_status_id'] : 0,
            'attributes_as_filters' => !empty($post['gincore_attributes_as_filters']) ? 1 : 0,
        ]);
        if(isset($post['admin_folder'])){
            $settings['admin_folder'] = $post['admin_folder'];
        }
        if(isset($post['catalog_folder'])){
            $settings['catalog_folder'] = $post['catalog_folder'];
        }
        
        return $settings;
    }
    
    /**
     * @param array $prepared_settings
     */
    public function savePreparedSettings($prepared_settings)
    {
        $this->load->model('setting/setting');
        $this->model_setting_setting->editSetting('gincore', ['gincore_settings' => $prepared_settings]);        
    }
    
    /**
     * @param array $post
     */
    public function saveSettingsFromPost($post)
    {
        $settings = $this->prepareSettingsFromPost($post);
        $this->savePreparedSettings($settings);
    }      
    
    /**
     * Create tables for integration
     */
    public function install()
    {
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_categories` (
                    `gincore_category_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_category_parent_id` BIGINT UNSIGNED NOT NULL DEFAULT '0' , 
                    `oc_category_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_category_id`), UNIQUE (`oc_category_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_products` (
                    `gincore_product_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_product_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_product_id`), UNIQUE (`oc_product_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_attributes` (
                    `gincore_attribute_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_attribute_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_filter_group_id` BIGINT UNSIGNED NULL DEFAULT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_attribute_id`), UNIQUE (`oc_attribute_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_attributes_sets` (
                    `gincore_attributes_set_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_attribute_group_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_attributes_set_id`), UNIQUE (`oc_attribute_group_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_attribute_options` (
                    `gincore_attribute_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_attribute_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_attribute_option_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_attribute_option_value` VARCHAR(255) NOT NULL , 
                    `oc_filter_id` BIGINT UNSIGNED NULL DEFAULT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_attribute_id`, `gincore_attribute_option_id`), 
                    INDEX (`oc_attribute_id`), INDEX (`oc_filter_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_product_attribute_options` (
                    `gincore_product_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_product_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_attribute_option_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_filter_id` BIGINT UNSIGNED NULL DEFAULT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    INDEX (`gincore_product_id`), INDEX (`oc_product_id`),
                    UNIQUE(`gincore_product_id`, `gincore_attribute_option_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_options` (
                    `gincore_attribute_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_option_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_attribute_id`), UNIQUE (`oc_option_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_option_values` (
                    `gincore_attribute_option_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_option_value_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_attribute_option_id`), UNIQUE (`oc_option_value_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_product_options` (
                    `gincore_product_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_product_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_attribute_option_id` BIGINT UNSIGNED NOT NULL , 
                    `oc_option_value_id` BIGINT UNSIGNED NOT NULL , 
                    `gincore_option_product_id` BIGINT UNSIGNED NOT NULL , 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    INDEX (`oc_product_id`), INDEX(`oc_option_value_id`),
                    UNIQUE(`gincore_product_id`, `gincore_attribute_option_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        $this->db->query("
            CREATE TABLE `" . DB_PREFIX . "gincore_orders` (
                    `gincore_order_id` BIGINT UNSIGNED NULL DEFAULT NULL, 
                    `oc_order_id` BIGINT UNSIGNED NOT NULL , 
                    `api_result` TEXT NULL DEFAULT NULL, 
                    `created_at` TIMESTAMP NULL DEFAULT NULL , 
                    `updated_at` TIMESTAMP NULL DEFAULT NULL ,
                    UNIQUE (`gincore_order_id`), UNIQUE (`oc_order_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
        ");
        
        if(strpos(VERSION, '2.') === 0){
            $this->db->query("INSERT INTO `" . DB_PREFIX . "event` SET `code` = 'gincore_send_order_to_api', `trigger` = '" . $this->db->escape('catalog/model/checkout/order/addOrderHistory/after') . "', `action` = '" . $this->db->escape('extension/module/gincore/orderCreated') . "', `status` = 1, `date_added` = now()");
            $this->db->query("INSERT INTO `" . DB_PREFIX . "event` SET `code` = 'gincore_customer_login', `trigger` = '" . $this->db->escape('catalog/model/account/customer/deleteLoginAttempts/after') . "', `action` = '" . $this->db->escape('extension/module/gincore/customerLoggedIn') . "', `status` = 1, `date_added` = now()");
        }else{ // >=3
            $this->db->query("INSERT INTO `" . DB_PREFIX . "event` SET `code` = 'gincore_send_order_to_api', `trigger` = '" . $this->db->escape('catalog/model/checkout/order/addOrderHistory/after') . "', `action` = '" . $this->db->escape('extension/module/gincore/orderCreated') . "', `sort_order` = 0, `status` = 1");
            $this->db->query("INSERT INTO `" . DB_PREFIX . "event` SET `code` = 'gincore_customer_login', `trigger` = '" . $this->db->escape('catalog/model/account/customer/deleteLoginAttempts/after') . "', `action` = '" . $this->db->escape('extension/module/gincore/customerLoggedIn') . "', `sort_order` = 0, `status` = 1");
        }
        
        // save default settings
        $this->saveSettingsFromPost([
            'admin_folder' => basename(DIR_APPLICATION),
            'catalog_folder' => basename(DIR_CATALOG),
            'gincore_attributes_as_filters' => 1
        ]); 
    }

    /**
     * Remove tables for integration
     */
    public function uninstall()
    {
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_categories`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_products`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_attributes`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_attributes_sets`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_attribute_options`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_product_attribute_options`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_options`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_option_values`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_product_options`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "gincore_orders`");
        
        $this->db->query("DELETE FROM `" . DB_PREFIX . "event` WHERE `code` = 'gincore_send_order_to_api'");
        $this->db->query("DELETE FROM `" . DB_PREFIX . "event` WHERE `code` = 'gincore_customer_login'");
        
        $this->db->query("DELETE FROM `" . DB_PREFIX . "setting` WHERE `code` = 'gincore'");
    }
}
