<?php

namespace gincore;

class Categories
{
    
    use models_loader;
    
    /**
     * @var array 
     */
    private $settings;
    /**
     * @var \Registry 
     */
    private $registry;
    /**
     * @var \DB 
     */
    private $db;
    /**
     * @var \ModelCatalogCategory
     */
    private $model_catalog_category;
    
    /**
     * @param array $settings
     * @param \Registry $registry
     */
    public function __construct($settings, $registry)
    {
        $this->settings = $settings;
        $this->registry = $registry;
        $this->db = $this->registry->get('db');
        
        $this->loadModels();
    }

    /**
     * 
     */
    private function loadModels()
    {
        $this->loadCategoryModel();
        $this->model_catalog_category = new \ModelCatalogCategory($this->registry);
    }
    
    /**
     * @param int $gincore_category_id
     * @param int $gincore_category_parent_id
     * @param int $oc_category_id
     */
    public function linkCategories($gincore_category_id, $gincore_category_parent_id, $oc_category_id)
    {
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "gincore_categories (gincore_category_id, gincore_category_parent_id, oc_category_id, created_at, updated_at)
            VALUES (" . ((int)$gincore_category_id) . ", " . ((int)$gincore_category_parent_id) . ", " . ((int)$oc_category_id) . ", NOW(), NOW())
            ON DUPLICATE KEY UPDATE updated_at = NOW(), gincore_category_parent_id = VALUES (gincore_category_parent_id)
        ");
    }
    
    /**
     * @return array
     */
    public function getLinkedCategories()
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "gincore_categories`");
	return $query->rows;
    }
    
    /**
     * @return int
     */
    public function getImportedCategoriesCount()
    {
        $query = $this->db->query("SELECT count(*) as c FROM `" . DB_PREFIX . "gincore_categories`");
	return $query->rows ? $query->rows[0]['c'] : 0;
    }
    
    /**
     * 
     */
    public function setParentIdForCategories()
    {
        $this->db->query("
            UPDATE " . DB_PREFIX . "category as c 
            LEFT JOIN " . DB_PREFIX . "gincore_categories as gc ON gc.oc_category_id = c.category_id
            SET c.parent_id = (SELECT oc_category_id FROM " . DB_PREFIX . "gincore_categories WHERE gincore_category_id = gc.gincore_category_parent_id)
        ");
    }
    
    /**
     * @param int $gincore_category_id
     * @return bool
     */
    public function hasCategory($gincore_category_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_categories as gc
            WHERE gincore_category_id = " . (int)$gincore_category_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "category WHERE category_id = gc.oc_category_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param array $category
     */
    public function createCategory($category)
    {
        $oc_category_id = $this->model_catalog_category->addCategory([
            'parent_id' => 0,
            'column' => 0,
            'sort_order' => 0,
            'status' => 1,
            'category_layout' => [0 => 0], // default
            'category_store' => [0], // default
            'category_description' => [
                $this->settings['language_id'] => [
                    'meta_title' => htmlspecialchars($category['name']),
                    'name' => htmlspecialchars($category['name']),
                    'description' => '',
                    'meta_keyword' => '',
                    'meta_description' => '',
                ]
            ]
        ]);
        $this->linkCategories($category['id'], $category['parent_id'], $oc_category_id);
    }
    
    /**
     * @param array $category
     */
    public function updateCategory($category)
    {
        $oc_category = $this->db->query("SELECT oc_category_id FROM " . DB_PREFIX . "gincore_categories WHERE gincore_category_id = " . (int)$category['id']);
        if(empty($oc_category->rows)){
            return;
        }
        $oc_category_id = $oc_category->rows[0]['oc_category_id'];
        $this->db->query("
            UPDATE " . DB_PREFIX . "category_description
            SET name = '" . $this->db->escape(htmlspecialchars($category['name'])) . "'
            WHERE category_id = " . $oc_category_id . " AND language_id = " . $this->settings['language_id'] . "
        ");
        $this->linkCategories($category['id'], $category['parent_id'], $oc_category_id);
    }
    
    /**
     * @param int $gincore_category_id
     */
    public function removeCategory($gincore_category_id)
    {
        $oc_category = $this->db->query("SELECT oc_category_id FROM " . DB_PREFIX . "gincore_categories WHERE gincore_category_id = " . (int)$gincore_category_id);
        if(empty($oc_category->rows)){
            return;
        }
        $oc_category_id = $oc_category->rows[0]['oc_category_id'];
        
        $query = $this->db->query("SELECT category_id FROM " . DB_PREFIX . "category_path WHERE path_id = '" . $oc_category_id . "'");
        $childrens = [];
        foreach ($query->rows as $result) {
            $childrens[] = $result['category_id'];
        }
        $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_categories WHERE oc_category_id IN (" . implode(',', $childrens) . ")");

        $this->model_catalog_category->deleteCategory($oc_category_id);
    }
        
    /**
     * 
     */
    public function removeCategories()
    {
        $linked_categories =  $this->getLinkedCategories();
        foreach($linked_categories as $category){
            $this->removeCategory($category['gincore_category_id']);
        }
    }
    
    /**
     * 
     */
    public function repairCategories()
    {
        $this->setParentIdForCategories();
        $this->model_catalog_category->repairCategories();
    }
}
