<?php

namespace gincore;

class Products
{
    
    use models_loader;
    
    /**
     * @var array 
     */
    private $settings;
    /**
     * @var \Registry 
     */
    private $registry;
    /**
     * @var \DB 
     */
    private $db;
    /**
     * @var \ModelCatalogProduct
     */
    private $model_catalog_product;
    /**
     * @var \ModelCatalogAttribute
     */
    private $model_catalog_attribute;
    /**
     * @var \ModelCatalogAttributeGroup
     */
    private $model_catalog_attribute_group;
    
    /**
     * @param array $settings
     * @param \Registry $registry
     */
    public function __construct($settings, $registry)
    {
        $this->settings = $settings;
        $this->registry = $registry;
        $this->db = $this->registry->get('db');
        
        $this->loadModels();
    }

    /**
     * 
     */
    private function loadModels()
    {
        $this->loadProductModel();
        $this->loadAttributeModel();
        $this->loadAttributeGroupModel();
        $this->loadFilterModel();
        $this->model_catalog_product = new \ModelCatalogProduct($this->registry);
        $this->model_catalog_attribute = new \ModelCatalogAttribute($this->registry);
        $this->model_catalog_attribute_group = new \ModelCatalogAttributeGroup($this->registry);
        $this->model_catalog_filter = new \ModelCatalogFilter($this->registry);
    }
    
    /**
     * @param int $gincore_product_id
     * @param int $oc_product_id
     */
    public function linkProducts($gincore_product_id, $oc_product_id)
    {
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "gincore_products (gincore_product_id, oc_product_id, created_at, updated_at)
            VALUES (" . ((int)$gincore_product_id) . ", " . ((int)$oc_product_id) . ", NOW(), NOW())
            ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_product_id = VALUES (oc_product_id)
        ");
    }
    
    /**
     * @return array
     */
    public function getLinkedProducts()
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "gincore_products`");
	return $query->rows;
    }
    
    /**
     * @return int
     */
    public function getImportedProductsCount()
    {
        $query = $this->db->query("SELECT count(*) as c FROM `" . DB_PREFIX . "gincore_products`");
	return $query->rows ? $query->rows[0]['c'] : 0;
    }
    
    /**
     * @param int $gincore_product_id
     * @return bool
     */
    public function hasProduct($gincore_product_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_products as gp
            WHERE gincore_product_id = " . (int)$gincore_product_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "product WHERE product_id = gp.oc_product_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_product_id
     * @return int|null
     */
    public function getOcProductId($gincore_product_id)
    {
        $oc_product = $this->db->query("SELECT oc_product_id FROM " . DB_PREFIX . "gincore_products WHERE gincore_product_id = " . (int)$gincore_product_id);
        if(empty($oc_product->rows)){
            return;
        }
        return $oc_product->rows[0]['oc_product_id'];
    }
    
    /**
     * @param int $gincore_option_product_id
     * @return int|null
     */
    public function getOcProductOption($gincore_option_product_id)
    {
        $oc_product_option = $this->db->query("SELECT * FROM " . DB_PREFIX . "gincore_product_options WHERE gincore_option_product_id = " . (int)$gincore_option_product_id);
        if(empty($oc_product_option->rows)){
            return;
        }
        return $oc_product_option->rows[0];
    }
    
    /**
     * @param string $image_url
     * @param int $image_id
     * @return string
     */
    public function uploadImage($image_url, $image_id)
    {
        $folder = 'catalog/g-images/';
        $dir = rtrim(DIR_IMAGE . $folder);
        if(!is_dir($dir)){
            mkdir($dir, 0777);
        }
        $filename = md5($image_id) . '.' . pathinfo($image_url, PATHINFO_EXTENSION);
        if(file_exists($dir . $filename)){
            return $folder . $filename; 
        }
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $image_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        $picture = curl_exec($ch);
        curl_close($ch);
        
        file_put_contents($dir . $filename, $picture);
        return $folder . $filename;
    }
    
    /**
     * @param array $gincore_categories
     * @return array
     */
    public function getOcCategories($gincore_categories)
    {
        $cats_query = $this->db->query("
            SELECT oc_category_id  FROM " . DB_PREFIX . "gincore_categories 
            WHERE gincore_category_id IN (" . implode(',', $gincore_categories ?: [0]) . ")
        ");
        $categories = [];
        foreach($cats_query->rows as $row){
            $categories[] = $row['oc_category_id'];
        }
        return $categories;
    }
    
    /**
     * 
     * @param array $gincore_product_attributes_set_id
     * @return bool
     */
    public function hasAttributeSet($gincore_product_attributes_set_id)
    {
        if(empty($gincore_product_attributes_set_id)){
            return false;
        }
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_attributes_sets as ats
            WHERE gincore_attributes_set_id = " . (int)$gincore_product_attributes_set_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "attribute_group WHERE attribute_group_id = ats.oc_attribute_group_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attributes_set_id
     * @return int
     */
    public function getOcAttributeGroupId($gincore_attributes_set_id)
    {
        $query = $this->db->query("
            SELECT oc_attribute_group_id  FROM " . DB_PREFIX . "gincore_attributes_sets 
            WHERE gincore_attributes_set_id = " . $gincore_attributes_set_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_attribute_group_id'];
        }
        return null;
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return bool
     */
    public function hasAttribute($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_attributes as att
            WHERE gincore_attribute_id = " . (int)$gincore_attribute_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "attribute WHERE attribute_id = att.oc_attribute_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return int
     */
    public function getOcAttributeId($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT oc_attribute_id  FROM " . DB_PREFIX . "gincore_attributes 
            WHERE gincore_attribute_id = " . $gincore_attribute_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_attribute_id'];
        }
        return null;
    }
    
    /**
     * @param array $gincore_product_attribute_option_value
     * @return bool
     */
    public function hasAttributeOption($gincore_product_attribute_option_value)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_attribute_options as ao
            WHERE gincore_attribute_option_id = " . (int)$gincore_product_attribute_option_value['attribute_option_id'] . "
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param array $gincore_attributes_set
     * @return int
     */
    public function makeAttributeGroup($gincore_attributes_set)
    {
        if(empty($gincore_attributes_set)){
            return null;
        }
        if(!$this->hasAttributeSet($gincore_attributes_set['id'])){
            $attribute_group = [
                'sort_order' => 0,
                'attribute_group_description' => [
                    $this->settings['language_id'] => [
                        'name' => $gincore_attributes_set['name']
                    ]
                ]
            ];
            $attribute_group_id = $this->model_catalog_attribute_group->addAttributeGroup($attribute_group);
        }else{
            $attribute_group_id = $this->getOcAttributeGroupId($gincore_attributes_set['id']);
            $this->db->query("UPDATE  " . DB_PREFIX . "attribute_group_description SET name = '" . $this->db->escape($gincore_attributes_set['name']) . "' WHERE attribute_group_id = " . $attribute_group_id . " AND language_id = " . $this->settings['language_id']);
        }
        return $attribute_group_id;
    }
    
    /**
     * @param array $gincore_attribute
     * @param int $attribute_group_id
     * @return int
     */
    public function makeAttribute($gincore_attribute, $attribute_group_id)
    {
        if(empty($gincore_attribute)){
            return null;
        }
        if(!$this->hasAttribute($gincore_attribute['attribute_id'])){
            $oc_attribute_id = $this->model_catalog_attribute->addAttribute([
                'attribute_group_id' => $attribute_group_id,
                'sort_order' => 0,
                'attribute_description' => [
                    $this->settings['language_id'] => [
                        'name' => $gincore_attribute['attribute_title']
                    ]
                ]
            ]);
        }else{
            $oc_attribute_id = $this->getOcAttributeId($gincore_attribute['attribute_id']);
            $this->db->query("UPDATE  " . DB_PREFIX . "attribute_description SET name = '" . $this->db->escape($gincore_attribute['attribute_title']) . "' WHERE attribute_id = " . $oc_attribute_id . " AND language_id = " . $this->settings['language_id']);
        }
        return $oc_attribute_id;
    }
    
    /**
     * @param array $product
     * @return array
     */
    public function makeProductAttributes($product)
    {
        if(empty($product['attributes']) || empty($product['attributes_set'])){
            return [];
        }
        $attributes = [];
        
        $attribute_group_id = $this->makeAttributeGroup($product['attributes_set']);
        foreach($product['attributes'] as $i => $attribute){
            $oc_attribute_id = $this->makeAttribute($attribute, $attribute_group_id);
            
            $values = [];
            $gincore_values = [];
            foreach($attribute['attribute_values'] as $attribute_value){
                $values[] = $attribute_value['attribute_option_value']; 
                $gincore_values[$attribute_value['attribute_option_id']] = $attribute_value['attribute_option_value'];
            }
            $attributes[] = [
                'attribute_id' => $oc_attribute_id, 
                'product_attribute_description' => [
                    $this->settings['language_id'] => [
                        'text' => implode(', ', $values)
                    ]
                ],
                'data' => [
                    'gincore_attribute_id' => $attribute['attribute_id'],
                    'gincore_values' => $gincore_values
                ]
            ];
        }
        
        return [
            'data' => [
                'gincore_attributes_set_id' => $product['attributes_set']['id'],
                'attribute_group_id' => $attribute_group_id
            ],
            'attributes' => $attributes
        ];
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return bool
     */
    public function hasFilterGroup($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_attributes as att
            WHERE gincore_attribute_id = " . (int)$gincore_attribute_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "filter_group WHERE filter_group_id = att.oc_filter_group_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return int
     */
    public function getOcFilterGroupId($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT oc_filter_group_id  FROM " . DB_PREFIX . "gincore_attributes 
            WHERE gincore_attribute_id = " . $gincore_attribute_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_filter_group_id'];
        }
        return null;
    }
    
    /**
     * @param array $gincore_attribute
     * @return int
     */
    public function makeFilterGroup($gincore_attribute)
    {
        if(empty($gincore_attribute)){
            return null;
        }
        if(!$this->hasFilterGroup($gincore_attribute['attribute_id'])){
            $this->db->query("INSERT INTO `" . DB_PREFIX . "filter_group` SET sort_order = 0");
            $oc_filter_group_id = $this->db->getLastId();
            $this->db->query("INSERT INTO " . DB_PREFIX . "filter_group_description SET filter_group_id = '" . (int)$oc_filter_group_id . "', language_id = '" . (int)$this->settings['language_id'] . "', name = '" . $this->db->escape($gincore_attribute['attribute_title']) . "'");
        }else{
            $oc_filter_group_id = $this->getOcFilterGroupId($gincore_attribute['attribute_id']);
            $this->db->query("UPDATE `" . DB_PREFIX . "filter_group_description` SET name = '" . $this->db->escape($gincore_attribute['attribute_title']) . "' WHERE language_id = " . $this->settings['language_id'] . " AND filter_group_id = " . $oc_filter_group_id);
        }
        return $oc_filter_group_id;
    }
    
    /**
     * @param int $gincore_attribute_option_id
     * @return bool
     */
    public function hasFilter($gincore_attribute_option_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_attribute_options as att
            WHERE gincore_attribute_option_id = " . (int)$gincore_attribute_option_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "filter WHERE filter_id = att.oc_filter_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attribute_option_id
     * @return int
     */
    public function getOcFilterId($gincore_attribute_option_id)
    {
        $query = $this->db->query("
            SELECT oc_filter_id  FROM " . DB_PREFIX . "gincore_attribute_options 
            WHERE gincore_attribute_option_id = " . $gincore_attribute_option_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_filter_id'];
        }
        return null;
    }
    
    /**
     * @param array $gincore_attribute_option
     * @param int $oc_filter_group_id
     * @return int
     */
    public function makeFilter($gincore_attribute_option, $oc_filter_group_id)
    {
        if(empty($gincore_attribute_option)){
            return null;
        }
        if(!$this->hasFilter($gincore_attribute_option['attribute_option_id'])){
            $this->db->query("INSERT INTO " . DB_PREFIX . "filter SET filter_group_id = '" . (int)$oc_filter_group_id . "', sort_order = 0");
            $oc_filter_id = $this->db->getLastId();
            $this->db->query("INSERT INTO " . DB_PREFIX . "filter_description SET filter_id = '" . (int)$oc_filter_id . "', language_id = '" . (int)$this->settings['language_id'] . "', filter_group_id = '" . (int)$oc_filter_group_id . "', name = '" . $this->db->escape($gincore_attribute_option['attribute_option_value']) . "'");
        }else{
            $oc_filter_id = $this->getOcFilterId($gincore_attribute_option['attribute_option_id']);
            $this->db->query("UPDATE " . DB_PREFIX . "filter_description SET name = '" . $this->db->escape($gincore_attribute_option['attribute_option_value']) . "' WHERE language_id = " . $this->settings['language_id'] . " AND filter_id = " . $oc_filter_id);
        }
        return $oc_filter_id;
    }
    
    /**
     * @param array $product
     * @return array
     */
    public function makeProductFilters($product)
    {
        if(empty($product['attributes']) || empty($product['attributes_set'])){
            return [];
        }
        $filters = $data = [];
        
        foreach($product['attributes'] as $i => $attribute){
            $oc_filter_group_id = $this->makeFilterGroup($attribute);
            
            $data_filters = [];
            foreach($attribute['attribute_values'] as $attribute_value){
                $oc_filter_id = $this->makeFilter($attribute_value, $oc_filter_group_id);
                $filters[] = $oc_filter_id; 
                $data_filters[$attribute_value['attribute_option_id']] = $oc_filter_id;
            }
            
            $data[$attribute['attribute_id']] = [
                'filter_group_id' => $oc_filter_group_id,
                'filters' => $data_filters
            ];
        }
        
        return [
            'filters' => $filters,
            'data' => $data
        ];
    }
    
    /**
     * @param array $oc_filters
     * @param array $oc_categories
     */
    public function addProductFiltersToCategories($oc_filters, $oc_categories)
    {
        if(empty($oc_filters['filters'])){
            return;
        }
        foreach ($oc_filters['filters'] as $oc_filter_id) {
            foreach ($oc_categories as $oc_category_id) {
                $this->db->query("INSERT IGNORE INTO " . DB_PREFIX . "category_filter SET category_id = " . $oc_category_id . ", filter_id = " . $oc_filter_id);
            }
        }
    }
    
    /**
     * @param array $product
     * @return array
     */
    public function makeProductImages($product)
    {
        $image = ''; $images = [];
        foreach($product['images'] as $i => $product_image){
            if($i === 0){
                $image = $this->uploadImage($product_image['url'], $product_image['id']);
                continue;
            }
            $images[] = [
                'image' => $this->uploadImage($product_image['url'], $product_image['id']),
                'sort_order' => $i - 1
            ];
        }
        return [
            'main' => $image,
            'images' => $images
        ];
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return bool
     */
    public function hasOption($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_options as opt
            WHERE gincore_attribute_id = " . (int)$gincore_attribute_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "option WHERE option_id = opt.oc_option_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attribute_id
     * @return int
     */
    public function getOcOptionId($gincore_attribute_id)
    {
        $query = $this->db->query("
            SELECT oc_option_id  FROM " . DB_PREFIX . "gincore_options 
            WHERE gincore_attribute_id = " . $gincore_attribute_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_option_id'];
        }
        return null;
    }
    
    /**
     * @param array $gincore_option
     * @return int
     */
    public function makeOption($gincore_option)
    {
        if(empty($gincore_option)){
            return null;
        }
        if(!$this->hasOption($gincore_option['attribute_id'])){
            $this->db->query("INSERT INTO `" . DB_PREFIX . "option` SET sort_order = 0, type = 'select'");
            $oc_option_id = $this->db->getLastId();
            $this->db->query("INSERT INTO " . DB_PREFIX . "option_description SET option_id = '" . (int)$oc_option_id . "', language_id = '" . (int)$this->settings['language_id'] . "', name = '" . $this->db->escape($gincore_option['attribute_title']) . "'");
        }else{
            $oc_option_id = $this->getOcOptionId($gincore_option['attribute_id']);
            $this->db->query("UPDATE `" . DB_PREFIX . "option_description` SET name = '" . $this->db->escape($gincore_option['attribute_title']) . "' WHERE language_id = " . $this->settings['language_id'] . " AND option_id = " . $oc_option_id);
        }
        return $oc_option_id;
    }
    
    /**
     * @param int $gincore_attribute_option_id
     * @return bool
     */
    public function hasOptionValue($gincore_attribute_option_id)
    {
        $query = $this->db->query("
            SELECT 1 FROM " . DB_PREFIX . "gincore_option_values as optv
            WHERE gincore_attribute_option_id = " . (int)$gincore_attribute_option_id . "
              AND EXISTS (SELECT 1 FROM " . DB_PREFIX . "option_value WHERE option_value_id = optv.oc_option_value_id)
        ");
        return !empty($query->rows);
    }
    
    /**
     * @param int $gincore_attribute_option_id
     * @return int
     */
    public function getOcOptionValueId($gincore_attribute_option_id)
    {
        $query = $this->db->query("
            SELECT oc_option_value_id  FROM " . DB_PREFIX . "gincore_option_values 
            WHERE gincore_attribute_option_id = " . $gincore_attribute_option_id . "
        ");
        if(!empty($query->rows)){
            return $query->rows[0]['oc_option_value_id'];
        }
        return null;
    }
    
    /**
     * @param array $gincore_product_option
     * @param int $oc_option_id
     * @return int
     */
    public function makeOptionValue($gincore_product_option, $oc_option_id)
    {
        if(empty($gincore_product_option)){
            return null;
        }
        if(!$this->hasOptionValue($gincore_product_option['attribute_option_id'])){
            $this->db->query("INSERT INTO " . DB_PREFIX . "option_value SET option_id = '" . $oc_option_id . "', image = '', sort_order = 0");
            $oc_option_value_id = $this->db->getLastId();
            $this->db->query("INSERT INTO " . DB_PREFIX . "option_value_description SET option_value_id = '" . $oc_option_value_id . "', language_id = '" . $this->settings['language_id'] . "', option_id = '" . $oc_option_id . "', name = '" . $this->db->escape($gincore_product_option['attribute_option_value']) . "'");
        }else{
            $oc_option_value_id = $this->getOcOptionValueId($gincore_product_option['attribute_option_id']);
            $this->db->query("UPDATE " . DB_PREFIX . "option_value_description SET name = '" . $this->db->escape($gincore_product_option['attribute_option_value']) . "' WHERE language_id = " . $this->settings['language_id'] . " AND option_value_id = " . $oc_option_value_id);
        }
        return $oc_option_value_id;
    }
    
    /**
     * @param array $product
     * @return array
     */
    public function makeProductOptions($product)
    {
        if(empty($product['options'])){
            return [];
        }
        $option_values = [];
        
        $total_free_balance = 0;
        $oc_option_id = $this->makeOption($product['options']);
        foreach($product['options']['products'] as $option){
            $oc_option_value_id = $this->makeOptionValue($option, $oc_option_id);
            $option_values[] = [
                'product_option_value_id' => null,
                'option_id' => $oc_option_id,
                'option_value_id' => $oc_option_value_id,
                'quantity' => $option['free_balance'],
                'subtract' => 1,
                'price' => $product['price'] <= $option['price'] ? $option['price'] - $product['price'] : $product['price'] - $option['price'],
                'price_prefix' => $product['price'] <= $option['price'] ? '+' : '-',
                'points' => 0,
                'points_prefix' => '',
                'weight' => 0,
                'weight_prefix' => '',
                'data' => [
                    'attribute_option_id' => $option['attribute_option_id'],
                    'gincore_option_product_id' => $option['id']
                ]
            ];
            $total_free_balance += $option['free_balance'];
        }
        if(!$option_values){
            return [];
        }
        
        $options = [
            'type' => 'select',
            'product_option_id' => null,
            'option_id' => $oc_option_id,
            'required' => 1,
            'product_option_value' => $option_values
        ];
        
        return [
            'options' => $options,
            'data' => [
                'attribute_id' => $product['options']['attribute_id'],
                'total_free_balance' => $total_free_balance
            ]
        ];
    }
    
    /**
     * @param array $attributes
     */
    public function linkAttributesSets($attributes)
    {
        if(empty($attributes['data'])){
            return;
        }
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "gincore_attributes_sets (gincore_attributes_set_id, oc_attribute_group_id, created_at, updated_at)
            VALUES (" . ((int)$attributes['data']['gincore_attributes_set_id']) . ", " . ((int)$attributes['data']['attribute_group_id']) . ", NOW(), NOW())
            ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_attribute_group_id = VALUES (oc_attribute_group_id)
        ");
    }
    
    /**
     * @param array $attributes
     * @param array $filters
     */
    public function linkAttributes($attributes, $filters)
    {
        if(empty($attributes['attributes'])){
            return;
        }
        foreach($attributes['attributes'] as $attribute){
            $this->db->query("
                INSERT INTO " . DB_PREFIX . "gincore_attributes (gincore_attribute_id, oc_attribute_id, oc_filter_group_id, created_at, updated_at)
                VALUES (" . $attribute['data']['gincore_attribute_id'] . ", " . $attribute['attribute_id'] . ", " . ($filters ? $filters['data'][$attribute['data']['gincore_attribute_id']]['filter_group_id'] : 'null') . ", NOW(), NOW())
                ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_attribute_id = VALUES (oc_attribute_id), oc_filter_group_id = VALUES (oc_filter_group_id)
            ");
        }
    }
    
    /**
     * @param array $attributes
     * @param array $filters
     */
    public function linkAttributeOptions($attributes, $filters)
    {
        if(empty($attributes['attributes'])){
            return;
        }
        foreach($attributes['attributes'] as $attribute){
            foreach($attribute['data']['gincore_values'] as $gincore_attribute_option_id => $gincore_attribute_option_value){
                $this->db->query("
                    INSERT INTO " . DB_PREFIX . "gincore_attribute_options (gincore_attribute_id, oc_attribute_id, gincore_attribute_option_id, gincore_attribute_option_value, oc_filter_id, created_at, updated_at)
                    VALUES (" . $attribute['data']['gincore_attribute_id'] . ", " . $attribute['attribute_id'] . ", " . $gincore_attribute_option_id . ", '" . $this->db->escape($gincore_attribute_option_value) . "', " . ($filters ? $filters['data'][$attribute['data']['gincore_attribute_id']]['filters'][$gincore_attribute_option_id] : 'null') . ", NOW(), NOW())
                    ON DUPLICATE KEY UPDATE updated_at = NOW(), gincore_attribute_option_value = VALUES(gincore_attribute_option_value), oc_attribute_id = VALUES(oc_attribute_id), oc_filter_id = VALUES(oc_filter_id)
                ");
            }
        }
    }
    
    /**
     * @param int $gincore_product_id
     * @param int $oc_product_id
     * @param array $attributes
     * @param array $filters
     * @param boolean $remove_unused = false
     */
    public function linkProductAttributeOptions($gincore_product_id, $oc_product_id, $attributes, $filters, $remove_unused = false)
    {
        $used = [];
        if(!empty($attributes['attributes'])){
            foreach($attributes['attributes'] as $attribute){
                foreach($attribute['data']['gincore_values'] as $gincore_attribute_option_id => $gincore_attribute_option_value){
                    $this->db->query("
                        INSERT INTO " . DB_PREFIX . "gincore_product_attribute_options (gincore_product_id, oc_product_id, gincore_attribute_option_id, oc_filter_id, created_at, updated_at)
                        VALUES (" . $gincore_product_id . ", " . $oc_product_id . ", " . $gincore_attribute_option_id . ", " . ($filters ? $filters['data'][$attribute['data']['gincore_attribute_id']]['filters'][$gincore_attribute_option_id] : 'null') . ", NOW(), NOW())
                        ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_filter_id = VALUES (oc_filter_id), oc_product_id = VALUES (oc_product_id)
                    ");
                    $used[] = $gincore_attribute_option_id;
                }
            }
        }
        if($remove_unused){
            $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_product_attribute_options WHERE gincore_attribute_option_id NOT IN (" . ($used ? implode(',', $used) : '0') . ") AND gincore_product_id = " . $gincore_product_id);
        }
    }
    
    /**
     * @param array $options
     */
    public function linkOptions($options)
    {
        if(empty($options['options'])){
            return;
        }
        $this->db->query("
            INSERT INTO " . DB_PREFIX . "gincore_options (gincore_attribute_id, oc_option_id, created_at, updated_at)
            VALUES (" . $options['data']['attribute_id'] . ", " . $options['options']['option_id'] . ", NOW(), NOW())
            ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_option_id = VALUES (oc_option_id)
        ");
        foreach($options['options']['product_option_value'] as $option_value){
            $this->db->query("
                INSERT INTO " . DB_PREFIX . "gincore_option_values (gincore_attribute_option_id, oc_option_value_id, created_at, updated_at)
                VALUES (" . $option_value['data']['attribute_option_id'] . ", " . $option_value['option_value_id'] . ", NOW(), NOW())
                ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_option_value_id = VALUES(oc_option_value_id)
            ");
        }
    }
    
    /**
     * @param int $gincore_product_id
     * @param int $oc_product_id
     * @param array $options
     * @param boolean $remove_unused = false
     */
    public function linkProductOptionValues($gincore_product_id, $oc_product_id, $options, $remove_unused = false)
    {
        $used = [];
        if(!empty($options['options'])){
            foreach($options['options']['product_option_value'] as $option_value){
                $this->db->query("
                    INSERT INTO " . DB_PREFIX . "gincore_product_options (gincore_product_id, oc_product_id, gincore_attribute_option_id, oc_option_value_id, gincore_option_product_id, created_at, updated_at)
                    VALUES (" . $gincore_product_id . ", " . $oc_product_id . ", " . $option_value['data']['attribute_option_id'] . ", " . $option_value['option_value_id'] . ", " . $option_value['data']['gincore_option_product_id'] . ", NOW(), NOW())
                    ON DUPLICATE KEY UPDATE updated_at = NOW(), oc_option_value_id = VALUES (oc_option_value_id), oc_product_id = VALUES (oc_product_id), gincore_option_product_id = VALUES (gincore_option_product_id)
                ");
                $used[] = $option_value['data']['attribute_option_id'];
            }
        }
        if($remove_unused){
            $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_product_options WHERE gincore_attribute_option_id NOT IN (" . ($used ? implode(',', $used) : '0') . ") AND gincore_product_id = " . $gincore_product_id);
        }
    }
    
    /**
     * @param array $product
     */
    public function createProduct($product)
    {
        $images = $this->makeProductImages($product);
        $attributes = $this->makeProductAttributes($product);
        $categories = $this->getOcCategories($product['categories']);
        
        $filters = [];
        if($this->settings['attributes_as_filters']){
            $filters = $this->makeProductFilters($product);
            $this->addProductFiltersToCategories($filters, $categories);
        }
        
        $options = $this->makeProductOptions($product);
        
        $oc_product_id = $this->model_catalog_product->addProduct([
            'model' => md5($product['id']),
            'sku' => htmlspecialchars($product['vendor_code']),
            'price' => $product['price'],
            'upc' => '',
            'ean' => '',
            'jan' => '',
            'isbn' => '',
            'mpn' => '',
            'location' => '',
            'quantity' => !empty($options['options']) ? $options['data']['total_free_balance'] : $product['free_balance'],
            'minimum' => 1,
            'subtract' => 1,
            'stock_status_id' => $this->settings['stock_status_id'],
            'shipping' => 1,
            'date_available' => date('Y-m-d H:i:s'),
            'manufacturer_id' => 0,
            'points' => 0,
            'weight' => 0,
            'weight_class_id' => 0,
            'length' => 0,
            'width' => 0,
            'height' => 0,
            'length_class_id' => 0,
            'status' => $product['available'] ? 1 : 0, // enabled
            'tax_class_id' => 0,
            'sort_order' => 1,
            'keyword' => '', // oc 2
            'product_layout' => [0 => 0], // default
            'product_store' => [0], // default
            'product_description' => [
                $this->settings['language_id'] => [
                    'name' => htmlspecialchars($product['title']),
                    'description' => htmlspecialchars($product['description']),
                    'tag' => '',
                    'meta_title' => htmlspecialchars($product['title']),
                    'meta_description' => '',
                    'meta_keyword' => ''
                ]
            ],
            'product_category' => $categories,
            'image' => $images['main'], // main
            'product_image' => $images['images'], // additional images
            'product_option' => $options ? [$options['options']] : [],
            'product_attribute' => $attributes ? $attributes['attributes'] : [],
            'product_filter' => $filters ? $filters['filters'] : [],
        ]);
        $this->linkProducts($product['id'], $oc_product_id);
        $this->linkAttributesSets($attributes);
        $this->linkAttributes($attributes, $filters);
        $this->linkAttributeOptions($attributes, $filters);
        $this->linkProductAttributeOptions($product['id'], $oc_product_id, $attributes, $filters);
        $this->linkOptions($options);
        $this->linkProductOptionValues($product['id'], $oc_product_id, $options);
    }
    
    /**
     * @param array $product
     */
    public function updateProduct($product)
    {
        $oc_product_id = $this->getOcProductId($product['id']);
        if(!$oc_product_id){
            return;
        }
        
        $images = $this->makeProductImages($product);
        $attributes = $this->makeProductAttributes($product);
        $categories = $this->getOcCategories($product['categories']);
        
        $filters = [];
        if($this->settings['attributes_as_filters']){
            $filters = $this->makeProductFilters($product);
            $this->addProductFiltersToCategories($filters, $categories);
        }
        
        $options = $this->makeProductOptions($product);
        
        $query = $this->db->query("SELECT DISTINCT * FROM " . DB_PREFIX . "product p WHERE p.product_id = '" . $oc_product_id . "'");
        $data = $query->row;
        $data['status'] = $product['available'] ? 1 : 0;
        $data['price'] = $product['price'];
        $data['sku'] = htmlspecialchars($product['vendor_code']);
        $data['quantity'] = !empty($options['options']) ? $options['data']['total_free_balance'] : $product['free_balance'];
        $data['product_description'] = $this->model_catalog_product->getProductDescriptions($oc_product_id);
        $data['product_description'][$this->settings['language_id']]['name'] = htmlspecialchars($product['title']);
        $data['product_description'][$this->settings['language_id']]['description'] = htmlspecialchars($product['description']);
        $data['product_category'] = $categories;
        $data['image'] = $images['main']; // main
        $data['product_image'] = $images['images']; // additional images
        $data['product_option'] = $options ? [$options['options']] : [];
        $data['product_attribute'] = $attributes ? $attributes['attributes'] : [];
        $data['product_filter'] = $filters ? $filters['filters'] : [];
        $data['product_layout'] = $this->model_catalog_product->getProductLayouts($oc_product_id);
        $data['product_store'] = $this->model_catalog_product->getProductStores($oc_product_id);
        
        $this->model_catalog_product->editProduct($oc_product_id, $data);
        
        $this->linkProducts($product['id'], $oc_product_id);
        $this->linkAttributesSets($attributes);
        $this->linkAttributes($attributes, $filters);
        $this->linkAttributeOptions($attributes, $filters);
        $this->linkProductAttributeOptions($product['id'], $oc_product_id, $attributes, $filters, true);
        $this->linkOptions($options);
        $this->linkProductOptionValues($product['id'], $oc_product_id, $options, true);
    }
    
    /**
     * @param int $gincore_product_id
     */
    public function removeProduct($gincore_product_id)
    {
        $oc_product_id = $this->getOcProductId($gincore_product_id);
        if(!$oc_product_id){
            return;
        }
        $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_products WHERE gincore_product_id = " . $gincore_product_id);
        $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_product_attribute_options WHERE gincore_product_id = " . $gincore_product_id);
        $this->db->query("DELETE FROM " . DB_PREFIX . "gincore_product_options WHERE gincore_product_id = " . $gincore_product_id);
        $this->model_catalog_product->deleteProduct($oc_product_id);
    }
        
    /**
     * 
     */
    public function removeProducts()
    {
        $linked_products =  $this->getLinkedProducts();
        foreach($linked_products as $product){
            $this->removeProduct($product['gincore_product_id']);
        }
    }
}
